--***************************************************************
--* File         : TB_WBSPIS.vhd                                *
--* Created      : 2009/09/22 10:38:10 kawasan                  *
--* Modified     : Time-stamp: <09/09/23 16:26:22 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2009 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_WBSPIS is end;

architecture behavior of TB_WBSPIS is
    
	component WBSPIS
	generic (
		REG_CTRL_INIT   : std_logic_vector(7 downto 0) := x"00"
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_ADR_I        : in    std_logic_vector(7 downto 0);
		WB_DAT_I        : in    std_logic_vector(7 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(7 downto 0);
		WB_IRQ_O        : out   std_logic;

		-- Other port interface:
		SPI_SS_N        : in    std_logic;	-- Active Low
		SPI_SCLK        : in    std_logic;	-- Serial Clock
		SPI_MOSI        : in    std_logic;	-- Master-Out Slave-In
		SPI_MISO        : out   std_logic;	-- Master-In Slave-Out

		SPI_SS          : out   std_logic;
		EVENT_START     : out   std_logic;
		EVENT_STOP      : out   std_logic;
		DONE            : out   std_logic
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz
	constant SPI_CYCLE      : Time := 250 ns;	-- 4MHz

	-- Address
	constant C_DATA         : std_logic_vector := x"00";
	constant C_CTRL         : std_logic_vector := x"01";
	constant C_STAT         : std_logic_vector := x"02";

	signal WB_CLK_I         : std_logic := '0';
	signal WB_RST_I         : std_logic := '1';
	signal WB_ADR_I         : std_logic_vector(7 downto 0) := (others=>'0');
	signal WB_DAT_I         : std_logic_vector(7 downto 0) := (others=>'0');
	signal WB_WE_I          : std_logic := '0';
	signal WB_STB_I         : std_logic := '0';
	signal WB_ACK_O         : std_logic := '0';
	signal WB_DAT_O         : std_logic_vector(7 downto 0) := (others=>'0');
	signal WB_IRQ_O         : std_logic := '0';

	signal SPI_S_N          : std_logic := '1';
	signal SPI_CLK          : std_logic := '1';
	signal SPI_DO           : std_logic := '0';
	signal SPI_DI           : std_logic := 'Z';

	signal SPI_SS           : std_logic := '0';
	signal EVENT_START      : std_logic := '0';
	signal EVENT_STOP       : std_logic := '0';
	signal DONE             : std_logic := '0';

	signal spi_s_n_old      : std_logic := '1';

	signal rd_data          : std_logic_vector(7 downto 0) := (others=>'0');
	signal rd_dat           : std_logic_vector(7 downto 0) := (others=>'0');
	signal wr_dat           : std_logic_vector(7 downto 0) := (others=>'0');

	signal spi_receive_tmp  : std_logic_vector(7 downto 0) := (others=>'0');
	signal spi_receive      : std_logic_vector(7 downto 0) := (others=>'0');

	-- WB Read (8bit)
	procedure rd_ub(
		ADD_I		: in    std_logic_vector(7 downto 0);
		DAT_I		: in    std_logic_vector(7 downto 0);
	signal	ADD_O		: out   std_logic_vector(7 downto 0);
	signal	DAT_O		: out   std_logic_vector(7 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		STB	<= '1';
		WE	<= '0';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		DAT_O	<= DAT_I;
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end rd_ub;

	-- WB Write (8bit)
	procedure wr_ub(
		ADD_I		: in    std_logic_vector(7 downto 0);
		DAT_I		: in    std_logic_vector(7 downto 0);
	signal	ADD_O		: out   std_logic_vector(7 downto 0);
	signal	DAT_O		: out   std_logic_vector(7 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		DAT_O	<= DAT_I;
		STB	<= '1';
		WE	<= '1';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		DAT_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end wr_ub;

begin
    
	uWBSPIS : WBSPIS
	generic map (
--		REG_CTRL_INIT   => x"01"
		REG_CTRL_INIT   => x"00"
	)
	port map (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        => WB_CLK_I,
		WB_RST_I        => WB_RST_I,
		WB_ADR_I        => WB_ADR_I,
		WB_DAT_I        => WB_DAT_I,
		WB_WE_I         => WB_WE_I,
		WB_STB_I        => WB_STB_I,
		WB_ACK_O        => WB_ACK_O,
		WB_DAT_O        => WB_DAT_O,
		WB_IRQ_O        => WB_IRQ_O,

		-- Other port interface:
		SPI_SS_N        => SPI_S_N,
		SPI_SCLK        => SPI_CLK,
		SPI_MOSI        => SPI_DI,
		SPI_MISO        => SPI_DO,

		SPI_SS          => SPI_SS,
		EVENT_START     => EVENT_START,
		EVENT_STOP      => EVENT_STOP,
		DONE            => DONE
	);

	process(WB_RST_I,WB_CLK_I)
	begin
		if WB_RST_I='1' then
			rd_dat <= (others=>'0');
		elsif WB_CLK_I'event and WB_CLK_I='1' then
			if WB_STB_I='1' and WB_WE_I='0' then
				rd_dat <= WB_DAT_O;
			else
				rd_dat <= rd_dat;
			end if;
		end if;
	end process;

	process begin
		wait for CLK_CYCLE/2;
		WB_CLK_I <= not WB_CLK_I;
	end process;

	process
		file VEC        : text is in "spi.vec";
		variable L      : line;
		variable CSEL   : std_logic;
		variable D_IN   : std_logic_vector(7 downto 0);
	begin
		wait for WT_IDLE*10;

		for iCnt2 in 0 to 10000 loop
			readline (VEC, L);
			read (L, CSEL);
			read (L, D_IN);

			SPI_S_N <= not CSEL;
			wait for WT_DELAY;
			if (SPI_S_N='0' and spi_s_n_old='1') or (SPI_S_N='1' and spi_s_n_old='0') then
				wait for SPI_CYCLE/2-WT_DELAY;
			end if;
			spi_s_n_old <= SPI_S_N;

			for iCnt in 7 downto 0 loop
				if SPI_S_N='0' then
					SPI_CLK <= '0';
					SPI_DI <= D_IN(iCnt);
					wait for SPI_CYCLE/2;
					SPI_CLK <= '1';
					wait for SPI_CYCLE/2;
				else
					SPI_CLK <= '1';
					SPI_DI <= 'Z';
					wait for SPI_CYCLE;
				end if;
			end loop;

			if endfile(VEC) then
				SPI_CLK <= '1';
				SPI_DI <= 'Z';
				wait;
			end if;
		end loop;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		WB_RST_I <= '0';
		wait for WT_IDLE;

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

	process begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		wait for WT_IDLE*2;
		wr_ub(C_CTRL, x"03", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
		rd_ub(C_CTRL, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- Ready
		wait until SPI_S_N'event and SPI_S_N='0';
		wait for WT_DELAY;

		-- Preset
		wr_ub(C_DATA, x"AA", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
		wait for SPI_CYCLE*4;
		wr_ub(C_DATA, x"12", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);

		-- SPI-Receive(WBSPIS)
		for iCnt in 7 downto 0 loop
			wait until DONE'event and DONE='1';
			wait for WT_DELAY;
			rd_ub(C_STAT, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
			rd_ub(C_DATA, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
			wr_dat <= not rd_dat;
			rd_ub(C_STAT, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
			wr_ub(C_DATA, wr_dat, WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
			rd_ub(C_STAT, WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
		end loop;
	end process;

	process(WB_RST_I,SPI_CLK)
	begin
		if WB_RST_I='1' then
			spi_receive_tmp <= (others=>'0');
		elsif SPI_CLK'event and SPI_CLK='1' then
			spi_receive_tmp <= spi_receive_tmp(6 downto 0) & SPI_DO;
		else
			spi_receive_tmp <= spi_receive_tmp;
		end if;
	end process;

	process(WB_RST_I,DONE)
	begin
		if WB_RST_I='1' then
			spi_receive <= (others=>'0');
		elsif DONE'event and DONE='1' then
			spi_receive <= spi_receive_tmp;
		else
			spi_receive <= spi_receive;
		end if;
	end process;

end behavior;
